// A poly data type demo program. Character differentiation.

// Gramma:
// Expression = Term { ( "+"|"-") Term  }
// Term =  Multiplier { "*"|"/" ) Multiplier }
// Multiplier = Factor { "^"  Factor }
// Factor = Number | "X" | "LN"  "(" Expression ")" | "(" Expression ")" "-" Factor 
// Number = Digit { Digit }
// Digit = "0"|"1"|"2"|"3"|"4"|"5"|"6"|"7"|"8"|"9"

program Differentiation; 
const 
  typeCHAR = GetTypeID('Char');
  typeINT  = GetTypeID('Integer');
  X       = 'X';
  OP_ADD  = '+';    
  OP_MULT = '*';    
  OP_SUB  = '-';     
  OP_DIV  = '/';
  OP_UN   = '~';   
  OP_LOG  = 'l';   
  OP_POW  = '^';   

function IsVar(const Term: Poly): boolean;
begin
  result := (PolyType(Term) = typeCHAR) and (Char(Term) = 'X');
end;

function IsConstant(const Term: Poly): boolean; 
begin
  result := PolyType(Term) = typeINT;
end;

function IsZero(const Term: Poly): boolean;
begin
  result:= IsConstant(Term) and (Integer(Term) = 0);
end;

function IsUnit(const Term: Poly): boolean;
begin
  result := IsConstant(Term) and (Integer(Term) = 1);
end;

function IsCompound(const Term:Poly): boolean;
begin
  result := Length(Term) > 0;
end;

function IsPow(const Term: Poly): boolean;
begin
  result := IsCompound(Term) and (Char(Term[1]) = OP_POW);
end;

function IsMult(const Term: Poly): boolean;
begin
  result := IsCompound(Term) and (Char(Term[1]) = OP_MULT);
end;

function IsAdd(const Term: Poly): boolean;
begin
  result := IsCompound(Term) and (Char(Term[1]) = OP_ADD);
end;

function IsSub(const Term: Poly): boolean;
begin
  result := IsCompound(Term) and (Char(Term[1]) = OP_SUB);
end;

function IsNeg(const Term: Poly): boolean;
begin
  result := IsCompound(Term) and (Char(Term[1]) = OP_UN);
end;

function Diff(const A : Poly) : Poly;
var
  K : Integer;
  X1, X2, R1, R2, R3 : Poly;
  Oper : Char;
begin
  K := Length(A);
  if K > 0 then   // array of poly         
  begin
    Oper := Char(A[1]);

    X1   == A[2];
    if K >= 3 then
      X2 == A[3];

    if      Oper = OP_ADD then
      result := [Oper, Diff(X1), Diff(X2)]  
    else if Oper = OP_SUB then 
      result := [Oper, Diff(X1), Diff(X2)]  
    else if Oper = OP_UN then
      result := [Oper, Diff(X1)]
    else if Oper = OP_MULT then
      result := [OP_ADD, [OP_MULT, Diff(X1), X2], [OP_MULT, X1, Diff(X2)]]     
    else if Oper = OP_DIV then
    begin 
      R1 := [OP_DIV, Diff(X1), X2]; 
      R2 := [OP_MULT, X1, Diff(X2)];
      R3 := [OP_MULT, X2, X2]; 
      result := [OP_SUB, R1, [OP_DIV, R2, R3]]; 
    end
    else if Oper = OP_LOG then
      result := [OP_DIV, Diff(X1), X1]  
    else if Oper = OP_POW then
    begin
      R1 := [OP_MULT, Diff(X1), [OP_MULT, X2, [OP_POW, X1, [OP_SUB, X2, 1]]]];   
      R2 := [OP_MULT, Diff(X2), [OP_MULT, [OP_LOG, X1], [OP_POW, X1, X2 ]]];       
      result := [OP_ADD, R1, R2];
    end
    else
      writeln('Syntax error !')
    end
  else if IsVar( A ) then
    result := 1
  else
    result := 0;
end;

function InOrder(const A: Poly): String;
var
  LL, RR, Oper : String;
begin    
  if IsCompound(A) then  
  begin
    if   Length(A) = 2 then  
    begin 
      Oper := Char(A[1]);
      if Oper = OP_UN then 
        Oper := OP_SUB;

      RR := InOrder(A[2]);

      if (Oper = OP_LOG) and (Copy(RR, 1, 1) <> '(') then  
        result :=  'ln' + '(' + RR + ')'
      else
        result := '(' + Oper + RR + ')';
    end
    else if Length(A) = 3 then
    begin
      Oper := Char(A[1]); 
      LL   := InOrder(A[2]);
      RR   := InOrder(A[3]);

      if (Copy(RR, 1, 1) = OP_SUB) and (Oper = OP_ADD) then
        Oper := '';

      result := '(' + LL + oper + RR + ')';
    end
  end
  else
    result := PolyToStr(A);
end;

procedure Compress(var R: Poly);
var
  K: Integer; 
  Oper: Char;
  X1, X2, U, V: Poly;
  Ground: boolean;
begin
  K := Length(R); 
  Oper := Char(R[1]);  
  if      K = 2 then
  begin
    if Oper = OP_UN then 
    begin
      X1 == R[2];
      if IsCompound(X1) then
        Compress(X1);      
      if IsConstant(X1) then
        R := -Integer(X1);
    end;
  end
  else if K = 3 then
  begin
    X1 == R[2];
    X2 == R[3];

    if IsCompound(X1) then
      Compress(X1);      
  
    if IsCompound(X2) then
      Compress(X2);

    Ground := IsConstant(X1) and IsConstant(X2);

    if Oper = OP_ADD then 
    begin 
      if Ground then
        R := Integer(X1) + Integer(X2)
      else if IsZero(X1) then
        R := X2
      else if IsZero(X2) then
        R := X1 
      else if X1 = X2 then 
        R := [OP_MULT, 2, X1] 
      else if IsNeg(X1) and (not IsNeg(X2)) then             
        R := [OP_SUB, X2, X1[2]]
      else if IsNeg(X2) and (not IsNeg(X1)) then              
        R := [OP_SUB, X1, X2[2]]
      else if IsMult(X1) and IsMult(X2) then
      begin
        if      X1[2] = X2[2] then
          R := [OP_MULT, X1[2], [OP_ADD, X1[3], X2[3]]]
        else if X1[3] = X2[3] then 
          R := [OP_MULT, X1[3], [OP_ADD, X1[2], X2[2]]];
      end
      else if IsMult(X1) then
      begin
        if      X1[2] = X2 then
           R := [OP_MULT, X2, [OP_ADD, X1[3], 1]]
        else if X1[3] = X2 then
           R := [OP_MULT, X2, [OP_ADD, X1[2], 1]];
      end
      else if IsMult(X2) then 
      begin
        if      X2[2] = X1 then
           R := [OP_MULT, X1, [OP_ADD, X2[3], 1]]
        else if X2[3] = X1 then
           R := [OP_MULT, X1, [OP_ADD, X2[2], 1]]; 
      end
      else if IsAdd(X1) and IsConstant(X2) then  
      begin
        if      IsConstant(X1[2]) then
           R := [OP_ADD, X1[3], Integer(X1[2]) + Integer(X2)]
        else if IsConstant(X1[3]) then
           R := [OP_ADD, X1[2], Integer(X1[3]) + Integer(X2)];  
      end
      else if IsAdd(X2) and IsConstant(X1) then   
      begin
        if      IsConstant(X2[2]) then
           R := [OP_ADD, X2[3], Integer(X2[2]) + Integer(X1)]
        else if IsConstant(X2[3]) then
           R := [OP_ADD, X2[2], Integer(X2[3]) + Integer(X1)];
      end
      else if IsSub(X1) and IsConstant(X2) then   
      begin
        if      IsConstant(X1[2]) then
           R := [OP_SUB, Integer(X1[2]) + Integer(X2), X1[3]]   
        else if IsConstant(X1[3]) then
           R := [OP_ADD, Integer(X2) - Integer(X1[3]), X1[2]];  
      end
      else if IsSub(X2) and IsConstant(X1) then    
      begin
        if      IsConstant(X2[2]) then
           R := [OP_SUB, Integer(X1) + Integer(X2[2]), X2[3]]
        else if IsConstant(X2[3]) then
           R := [OP_ADD, Integer(X1) - Integer(X2[3]), X2[2]];
      end
    end
    else if Oper = OP_SUB then  
    begin
      if Ground  then
        R := Integer(X1) - Integer(X2)
      else if IsZero(X1) then
        R := [OP_UN, X2]
      else if IsZero(X2) then
        R := X1 
      else if X1 = X2 then
        R := 0  
      else if IsNeg(X1) and (not IsNeg(X2)) then              
        R := [OP_ADD, X1[2], X2]
      else if IsNeg(X2) and (not IsNeg(X1)) then              
        R := [OP_ADD, X1, X2[2]] 
      else if IsMult(X1) and IsMult(X2) then 
      begin
        if      X1[2] = X2[2] then
          R := [OP_MULT, X1[2], [OP_SUB, X1[3], X2[3]]] 
        else if X1[3] = X2[3] then 
          R := [OP_MULT, X1[3], [OP_SUB, X1[2], X2[2]]]; 
      end
      else if IsMult(X1) then
      begin
        if      X1[2] = X2 then
           R := [OP_MULT, X2, [OP_SUB, X1[3], 1]]
        else if X1[3] = X2 then
           R := [OP_MULT, X2, [OP_SUB, X1[2], 1]]; 
      end
      else if IsMult(X2) then 
      begin
        if      X2[2] = X1 then
           R := [OP_MULT, X1, [OP_SUB, X2[3], 1]]
        else if X2[3] = X1 then
           R := [OP_MULT, X1, [OP_SUB, X2[2], 1]]; 
      end
      else if IsAdd(X1) and IsConstant(X2) then   
      begin
        if      IsConstant(X1[2]) then
           R := [OP_ADD, X1[3], Integer(X1[2]) - Integer(X2)] 
        else if IsConstant(X1[3]) then
           R := [OP_ADD, X1[2], Integer(X1[3]) - Integer(X2)];  
      end
      else if IsAdd(X2) and IsConstant(X1) then   
      begin
        if      IsConstant(X2[2]) then
           R := [OP_ADD, X2[3], Integer(X1) - Integer(X2[2])] 
        else if IsConstant( X2[ 3 ] ) then
           R := [OP_ADD, X2[2], Integer(X1) - Integer(X2[3])];  
      end
      else if IsSub(X1) and IsConstant(X2) then    
      begin
        if      IsConstant(X1[2]) then
           R := [OP_SUB, Integer(X1[2]) - Integer(X2), X1[3]]
        else if IsConstant(X1[3]) then
           R := [OP_SUB, X1[2], Integer(X2) + Integer(X1[3])];  
      end
      else if IsSub(X2) and IsConstant(X1) then    
      begin
        if      IsConstant(X2[2]) then
           R := [OP_ADD, Integer(X1) - Integer(X2[2]), X2[3]]
        else if IsConstant(X2[3]) then
           R := [OP_SUB, Integer(X1) - Integer(X2[3]), X2[2]];                            
      end
    end
    else if Oper = OP_MULT then 
    begin
      if Ground  then
        R := Integer(X1) * Integer(X2)
      else if IsZero(X1) then
        R := 0
      else if IsZero(X2) then
        R := 0
      else if IsNeg(X1) then
        R := [OP_MULT, -1, [OP_MULT, X1[2], X2]]
      else if IsNeg(X2) then 
        R := [OP_MULT, -1, [OP_MULT, X2[2], X1]]
      else if IsUnit(X1) then
        R := X2
      else if IsUnit(X2) then
        R := X1
      else if X1 = X2 then 
        R := [OP_POW, X1, 2]
      else if IsPow(X1) and IsPow(X2) and (X1[2] = X2[2]) then
        R := [OP_POW, X1[2], [OP_ADD, X1[3], X2[3]]]
      else if IsPow(X1) and (X1[2] = X2) then
        R := [OP_POW, X1[2], [OP_ADD, X1[3], 1]] 
      else if IsPow(X2) and (X2[2] = X1) then 
        R := [OP_POW, X2[2], [OP_ADD, X2[3], 1]] 
      else if IsMult(X1) and IsPow(X2) then
      begin
        U == X2[ 2 ];                          // ( A*U ) * ( U^V )
        V == X2[ 3 ];
        if      X1[2] = U then
          R := [OP_MULT, X1[3], [OP_POW, U, [OP_ADD, V, 1]]]      
        else if X1[3] = U then
          R := [OP_MULT, X1[2], [OP_POW, U, [OP_ADD, V, 1]]]      
        else if IsPow(X1[2]) and (X1[2][2] = U) then 
  
          // A == X1[ 3 ]
          // U == X1[ 2, 2 ]
          // C == X1[ 2, 3 ]
          // ( U ^ C )*A  *  ( U ^ V ) = A * ( U ^ ( C + V ) )
  
          R := [OP_MULT, X1[3], [OP_POW, U, [OP_ADD, V, X1[2][3]]]]   
        else if IsPow(X1[3]) and (X1[3][2] = U) then  
  
          // A == X1[ 2 ]
          // U == X1[ 3, 2 ] 
          // C == X1[ 3, 3 ] 
          // A*( U ^ C )  *  ( U ^ V ) = A * ( U ^ ( C + V ) ) 
  
          R := [OP_MULT, X1[2], [OP_POW, U, [OP_ADD, V, X1[3][3]]]];   
      end
      else if IsMult(X2) and IsPow(X1) then 
      begin
        U == X1[ 2 ];                           // symmetry 
        V == X1[ 3 ];
        if    X2[2] = U then
          R := [OP_MULT, X2[3], [OP_POW, U, [OP_ADD, V, 1]]]      
        else if X2[3] = U then
          R := [OP_MULT, X2[2], [OP_POW, U, [OP_ADD, V, 1]]]      
        else if IsPow(X2[2]) and (X2[2][2] = U) then  
  
          // A == X2[ 3 ]
          // U == X2[ 2, 2 ]
          // C == X2[ 2, 3 ]
          // ( U ^ C )*A  *  ( U ^ V ) = A * ( U ^ ( C + V ) )
  
          R := [OP_MULT, X2[3], [OP_POW, U, [OP_ADD, V, X2[2][3]]]]   
        else if IsPow(X2[3]) and (X2[3][2] = U) then  
  
          // A == X2[ 2 ]
          // U == X2[ 3, 2 ] 
          // C == X2[ 3, 3 ] 
          // A*( U ^ C )  *  ( U ^ V ) = A * ( U ^ ( C + V ) ) 
  
          R := [OP_MULT, X2[2], [OP_POW, U, [OP_ADD, V, X2[3][3]]]];   
      end
      else if IsMult(X1) then
      begin
        if      X1[2] = X2 then
          R := [OP_MULT, X1[3], [OP_POW, X2, 2]]
        else if X1[3] = X2 then
          R := [OP_MULT, X1[2], [OP_POW, X2, 2 ]] 
        else if IsPow(X1[2]) and (X1[2,2] = X2) then
          R := [OP_MULT, X1[3], [OP_POW, X2, [OP_ADD, X1[2, 3], 1]]]               
        else if IsPow(X1[3]) and (X1[3,2] = X2) then 
          R := [OP_MULT, X1[2], [OP_POW, X2, [OP_ADD, X1[3, 3], 1]]];               
      end  
      else if IsMult(X2) then 
      begin
        if      X2[2] = X1 then
          R := [OP_MULT, X2[3], [OP_POW, X1, 2]]
        else if X2[3] = X1 then
          R := [OP_MULT, X2[2], [OP_POW, X1, 2]] 
        else if IsPow(X2[2]) and (X2[2, 2] = X1) then 
          R := [OP_MULT, X2[3], [OP_POW, X1, [OP_ADD, X2[2, 3], 1]]]               
        else if IsPow(X2[3]) and (X2[3, 2] = X1) then 
          R := [OP_MULT, X2[2], [OP_POW, X1, [OP_ADD, X2[3, 3], 1]]];               
      end;
    end
    else if Oper = OP_DIV then 
    begin 
      if IsUnit(X2) then
        R := X1
      else if IsZero(X1) then
        R := X1
      else if IsPow(X2) then  
        R := [OP_MULT, X1, [OP_POW, X2[2], [OP_UN, X2[3]]]]  
      else if IsZero(X2) then
        writeln( 'Division by zero' )
      else
        R := [OP_MULT, X1, [OP_POW, X2, -1]];
    end
    else if Oper = OP_POW then
    begin
      if      IsUnit(X2) then
        R := X1
      else if IsZero(X2) then
        R := 1;
    end; 
  end;
end;

var
  E : Poly;
begin
  E := ['^',2,X];         

  writeln('Source expression in prefix notation:');
  writeln(E);

  writeln('Source expression in infix notation:');
  writeln(InOrder(E));   

  E := Diff(E);         

  writeln('Derivation in prefix notation:');
  writeln(E);           

  writeln('Derivation in infix notation:');
  writeln(InOrder(E)); 

  Compress(E);          

  writeln('Finally result in prefix notation:');
  writeln(E);  

  writeln('Finally result in infix notation:');
  writeln(InOrder(E));  
end.

