// Gauss algorithm: an open array parameters demo program
// 

program GaussDemo;

type
  TVector = array of Real;
  TMatrix = array of array of Real;

procedure Gauss( var A : TMatrix;
                 var B : TVector;
                 var X : TVector;
                 N : integer;
                 var error : boolean );

var
 I, J, I1, K, L : Integer;
 hold, sum, ab, t, big : Real;
begin
  error := false;

  for I:=1 to N-1 do
  begin
    big := Abs( A[I,I]);

    L := I;
    I1:= I+1;

    for J:=I1 to N do
    begin  // search for largest element 
     ab := Abs( A[J,I] );
     if ab > big then
     begin
       big := ab;
       L := J
     end;
   end;

  if big = 0.0 then error:= true
  else
    begin
      if L <> I then
      begin
        // interchange rows to put largest element on diagonal 
        for J:=1 to N do
        begin
          hold := A[ L, J ];
          A[ L, J ]:= A[ I, J ];
          A[ I, J ]:= hold;
        end;
        hold   := B[ L ];
        B[ L ] := B[ I ];
        B[ I ] := hold;
      end;

      for J:=I1 to N do
      begin
        t := A[ J, I ] / A[ I, I ];
        for K:=1 to N do
          A[ J, K ] := A[ J, K ] - t * A[ I, K ];
        B[ J ] := B[ J ] - t* B[ I ];
      end;
    end;
  end;

  if A[ N, N ] = 0.0 then error := true
  else
   begin
     X[ N ] := B[ N ] / A[ N, N ];
     I := N - 1;
     // back substitution
     repeat
       sum :=0.0;
       for J:=I+1 to N do
         sum := sum + A[ I, J ] * X[ J ];
       X[ I ] :=( B[ I ] - sum ) / A[ I, I ];
       I := I - 1;
     until I = 0;
   end;
end;

procedure Init(  var A : TMatrix;
                 var B : TVector;
                 var X : TVector;
                 N : integer );
var
  I, J : Integer;
begin
  for I:=1 to N do
  for J:=1 to N do
  begin
    A[ I, J ] := Random( 100 ) + 1;
    B[ I ] := Random( 100 ) + 1;
    X[ I ] := 0;
  end;
end;

procedure Show(  var A : TMatrix;
                 var B : TVector;
                 var X : TVector;
                 N : integer );
var
  I, J : Integer;
  R : Real;
begin
  writeln( '' );
  for I:=1 to N do
  begin
    R := 0;
    writeln( '' );
    for J:=1 to N do
    begin
      write( A[ I, J ]:10:2, ' ' );
      R := R + A[ I, J ] * X[ J ];
    end;
    write( '       ', B[ I ]:10:2 );
    write( '       ', R:10:2 );
  end;
  writeln( '' );
  writeln( '' );
  for J:=1 to N do
    write( X[ J ]:10:2 );
end;

const
  N = 4;
var
  A : array[ 1..10, 1..10 ] of Real;
  B, X : array[ 1..10 ] of Real;
  Error : boolean;
begin
  Init( A, B, X, N );

  Show( A, B, X, N );

  Gauss( A, B, X, N, Error );

  Show( A, B, X, N );
end.
