// Classses and Objects.
//

unit Shapes;

interface

type
  TShape = class
             destructor Free; virtual;
             procedure Show; virtual;
             function Norm : Integer; virtual;
           end; 

  TPoint = class( TShape )
           private
             FX, FY : Integer;
             procedure SetX( IX : Integer ); 
             procedure SetY( IY : Integer );
           public
             constructor Create( IX, IY : Integer );
             function Norm : Integer; override;
             procedure Show; override;
             property X : Integer read FX write SetX; 
             property Y : Integer read FY write SetY; 
           end;

  TCircle = class( TPoint )
            private
              FR : Integer;
              procedure SetR( IR : Integer );
            public
              constructor Create( IX, IY, IR : Integer );
              procedure Show; override;
              function Norm : Integer; override;
              property R : Integer read FR write SetR; 
            end; 

  TSquare = class( TShape )
            private
              P1, P2, C : TPoint;
              function GetPoint( Index : Integer ) : TPoint;
              procedure SetPoint( Index : Integer; Value : TPoint );
              function GetCenter : TPoint; 
            public  
              constructor Create( IP1, IP2 : TPoint );
              procedure Show; override;
              destructor Free; override;
              property Points[ I : Integer ] : TPoint read GetPoint 
                                                 write SetPoint;
              property Center : TPoint read GetCenter;
            end;

implementation

destructor TShape.Free; 
begin
end;                         

procedure TShape.Show;
begin
end;

function TShape.Norm : Integer;
begin
  result := 0;
end;

constructor TPoint.Create( IX, IY : Integer );
begin
  FX := IX;
  FY := IY;
end;

function TPoint.Norm : Integer;
begin
  result := FX * FX + FY * FY;
end;

procedure TPoint.Show;
begin
  writeln( 'Point:' );
  writeln( 'X=', FX, ',Y=', FY );
  writeln( 'End of Point' );
end;

procedure TPoint.SetX( IX : Integer );
begin
  writeln( 'You have changed X-coordinate' );
  FX := IX;
end;

procedure TPoint.SetY( IY : Integer );
begin
  writeln( 'You have changed Y-coordinate' );
  FY := IY;
end;

constructor TCircle.Create( IX, IY, IR : Integer );
begin
  inherited Create( IX, IY );
  FR := IR;
end;

procedure TCircle.Show;
begin
  writeln( 'Circle:' );
  inherited Show;
  writeln( 'R=', FR );
  writeln( 'End of Circle' );
end;

procedure TCircle.SetR( IR : Integer );
begin
  writeln( 'You have changed radius' );
  FR := IR;
end;

function TCircle.Norm : Integer;
begin
  result := FR * FR + inherited Norm;
end;

constructor TSquare.Create( IP1, IP2 : TPoint );
begin
  P1 := IP1;
  P2 := IP2;
end;

function TSquare.GetPoint( Index : Integer ) : TPoint;
begin
  if      Index = 1 then
    result := P1
  else if Index = 2 then
    result := P2
  else
    writeln( 'Incorrect index value !' );
end;

procedure TSquare.SetPoint( Index : Integer; Value : TPoint );
begin
  if      Index = 1 then
  begin
    if Assigned( P1 ) then
      P1.Free;
    P1 := Value;
  end
  else if Index = 2 then
  begin
    if Assigned( P2 ) then
      P2.Free;
    P2 := Value;
  end
  else
    writeln( 'Incorrect index value !' );
end;

procedure TSquare.Show;
begin
  writeln;
  writeln( 'Square:' );
  P1.Show;
  P2.Show;
  writeln( 'End of Square' );
  writeln;
end;

function TSquare.GetCenter : TPoint; 
var
  I : Integer;
begin
  if not Assigned( C ) then
    C := TPoint.Create( 0, 0 );

  C.FX := ( P1.FX + P2.FX ) div 2;
  C.FY := ( P1.FY + P2.FY ) div 2;
  result := C; 
end;

destructor TSquare.Free;
begin
  P1.Free;
  P2.Free;

  if Assigned( C ) then
    C.Free;

  inherited;
end;

end.
